@@ -6,7 +6,7 @@ import logging
 import sys
 from collections import OrderedDict
 from copy import deepcopy
-from datetime import datetime, date
+from datetime import datetime
 from os.path import join
 from typing import List, Union, Optional, Dict, Any, Tuple, Callable, Iterator, Iterable, Set
 
@@ -391,43 +391,43 @@ class Dataset(HDXObject):
                 ignore_fields = ['package_id']
                 resource.check_required_fields(ignore_fields=ignore_fields)
 
-    # @staticmethod
-    # def revise(match, filter=dict(), update=dict(), files_to_upload=dict(), configuration=None):
-    #     # type: (Dict, Dict, Dict, Dict, Optional[Configuration]) -> Optional['Dataset']
-    #     """Creates or updates an HDX object in HDX, saving current data and replacing with returned HDX object data
-    #     from HDX
-    #
-    #     Args:
-    #         match (Dict): Metadata on which to match dataset
-    #         filter (Dict): Filters to apply
-    #         update (Dict): Metadata updates to apply
-    #         files_to_upload (Dict): Files to upload to HDX
-    #         configuration (Optional[Configuration]): HDX configuration. Defaults to global configuration.
-    #
-    #     Returns:
-    #         Optional[Dataset]: Dataset object if successful revise, None if not
-    #     """
-    #     data = {'match': match}
-    #     if filter:
-    #         data['filter'] = filter
-    #     if update:
-    #         data['update'] = update
-    #     dataset = Dataset(data, configuration=configuration)
-    #     result = dataset._write_to_hdx('revise', data, id_field_name='match', files_to_upload=files_to_upload)
-    #     if result:
-    #         dataset.data = result['package']
-    #         dataset.separate_resources()
-    #         return dataset
-    #     return None
-    #
-    def _save_dataset_add_filestore_resources(self, default_operation, id_field_name, filestore_resources, hxl_update, create_default_views=False, **kwargs):
-        # type: (str, str, List[hdx.data.resource.Resource], bool, bool, Any) -> None
+    @staticmethod
+    def revise(match, filter=list(), update=dict(), files_to_upload=dict(), configuration=None):
+        # type: (Dict, List, Dict, Dict, Optional[Configuration]) -> 'Dataset'
+        """Revises an HDX dataset in HDX
+
+        Args:
+            match (Dict): Metadata on which to match dataset
+            filter (List): Filters to apply
+            update (Dict): Metadata updates to apply
+            files_to_upload (Dict): Files to upload to HDX
+            configuration (Optional[Configuration]): HDX configuration. Defaults to global configuration.
+
+        Returns:
+            Dataset: Dataset object
+        """
+        separators = (',', ':')
+        data = {'match': json.dumps(match, separators=separators)}
+        if filter:
+            data['filter'] = json.dumps(filter, separators=separators)
+        if update:
+            data['update'] = json.dumps(update, separators=separators)
+        dataset = Dataset(data, configuration=configuration)
+        result = dataset._write_to_hdx('revise', data, id_field_name='match', files_to_upload=files_to_upload)
+        dataset.data = result['package']
+        dataset.init_resources()
+        dataset.separate_resources()
+        return dataset
+
+    def _save_dataset_add_filestore_resources(self, default_operation, id_field_name, resources_to_delete, filestore_resources, hxl_update, create_default_views=False, **kwargs):
+        # type: (str, str, List[int], Dict[int, str], bool, bool, Any) -> None
         """Helper method to save the modified dataset and add any filestore resources
 
         Args:
             default_operation (str): Operation to perform eg. patch. Defaults to update.
             id_field_name (str): Name of field containing HDX object identifier
-            filestore_resources (List[hdx.data.resource.Resource]): List of resources that use filestore (to be appended to)
+            resources_to_delete (List[int]): List of indexes of resources to delete
+            filestore_resources (Dict[int, str]): List of (index of resources, file to upload)
             hxl_update (bool): Whether to call package_hxl_update.
             create_default_views (bool): Whether to create default views. Defaults to False.
             **kwargs: See below
@@ -436,7 +436,10 @@ class Dataset(HDXObject):
         Returns:
             None
         """
-
+        for i in sorted(resources_to_delete, reverse=True):
+            del self.resources[i]
+        if self.resources:
+            self.data['resources'] = self._convert_hdxobjects(self.resources)
         self.clean_tags()
         scriptinfo = kwargs.get('updated_by_script')
         if scriptinfo:
@@ -463,9 +466,10 @@ class Dataset(HDXObject):
             del kwargs['operation']
         if not existing_ignore_check and default_operation == 'create':
             del kwargs['ignore_check']
-        hdx.data.filestore_helper.FilestoreHelper.add_filestore_resources(self.data['resources'], filestore_resources, **kwargs)
         self.init_resources()
         self.separate_resources()
+        hdx.data.filestore_helper.FilestoreHelper.add_filestore_resources(self.resources, filestore_resources, **kwargs)
+
         if create_default_views:
             self.create_default_views()
         self._create_preview_resourceview()
@@ -492,7 +496,8 @@ class Dataset(HDXObject):
         if 'resources' in self.data:
             del self.data['resources']
         updated_resources = self.old_data.get('resources', None)
-        filestore_resources = list()
+        resources_to_delete = list()
+        filestore_resources = dict()
         if update_resources and updated_resources:
             ignore_fields = ['package_id']
             if match_resources_by_metadata:
@@ -501,22 +506,18 @@ class Dataset(HDXObject):
                     resource = self.resources[resource_index]
                     updated_resource = updated_resources[updated_resource_matches[i]]
                     logger.warning('Resource exists. Updating %s' % resource['name'])
-                    hdx.data.filestore_helper.FilestoreHelper.dataset_merge_filestore_resource(resource, updated_resource,
-                                                                                               filestore_resources,
-                                                                                               ignore_fields, **kwargs)
+                    hdx.data.filestore_helper.FilestoreHelper.dataset_merge_filestore_resource(
+                        resource, updated_resource, filestore_resources, resource_index, ignore_fields, **kwargs)
                 for resource_index in updated_resource_no_matches:
                     updated_resource = updated_resources[resource_index]
-                    hdx.data.filestore_helper.FilestoreHelper.check_filestore_resource(updated_resource, ignore_fields,
-                                                                                       filestore_resources, **kwargs)
+                    hdx.data.filestore_helper.FilestoreHelper.check_filestore_resource(
+                        updated_resource, ignore_fields, filestore_resources, resource_index, **kwargs)
                     self.resources.append(updated_resource)
                 if remove_additional_resources:
-                    resources_to_delete = list()
                     for resource_index in resource_no_matches:
                         resource = self.resources[resource_index]
                         logger.warning('Removing additional resource %s!' % resource['name'])
                         resources_to_delete.append(resource_index)
-                    for i in sorted(resources_to_delete, reverse=True):
-                        del self.resources[i]
             else:  # update resources by position
                 for i, updated_resource in enumerate(updated_resources):
                     if len(self.resources) > i:
@@ -526,24 +527,20 @@ class Dataset(HDXObject):
                         logger.warning('Resource exists. Updating %s' % resource_name)
                         if resource_name != updated_resource_name:
                             logger.warning('Changing resource name to: %s' % updated_resource_name)
-                        hdx.data.filestore_helper.FilestoreHelper.dataset_merge_filestore_resource(resource, updated_resource,
-                                                                         filestore_resources, ignore_fields, **kwargs)
+                        hdx.data.filestore_helper.FilestoreHelper.dataset_merge_filestore_resource(
+                            resource, updated_resource, filestore_resources, i, ignore_fields, **kwargs)
                     else:
-                        hdx.data.filestore_helper.FilestoreHelper.check_filestore_resource(updated_resource, ignore_fields, filestore_resources, **kwargs)
+                        hdx.data.filestore_helper.FilestoreHelper.check_filestore_resource(
+                            updated_resource, ignore_fields, filestore_resources, i, **kwargs)
                         self.resources.append(updated_resource)
 
                 if remove_additional_resources:
-                    resources_to_delete = list()
                     for i, resource in enumerate(self.resources):
                         if len(updated_resources) <= i:
                             logger.warning('Removing additional resource %s!' % resource['name'])
                             resources_to_delete.append(i)
-                    for i in sorted(resources_to_delete, reverse=True):
-                        del self.resources[i]
 
-        if self.resources:
-            self.data['resources'] = self._convert_hdxobjects(self.resources)
-        self._save_dataset_add_filestore_resources('update', 'id', filestore_resources, hxl_update,
+        self._save_dataset_add_filestore_resources('update', 'id', resources_to_delete, filestore_resources, hxl_update,
                                                    create_default_views=create_default_views, **kwargs)
 
     def update_in_hdx(self, update_resources=True, match_resources_by_metadata=True,
@@ -623,13 +620,12 @@ class Dataset(HDXObject):
             logger.info('Updated %s' % self.get_hdx_url())
             return
 
-        filestore_resources = list()
+        filestore_resources = dict()
         if self.resources:
             ignore_fields = ['package_id']
-            for resource in self.resources:
-                hdx.data.filestore_helper.FilestoreHelper.check_filestore_resource(resource, ignore_fields, filestore_resources)
-            self.data['resources'] = self._convert_hdxobjects(self.resources)
-        self._save_dataset_add_filestore_resources('create', 'name', filestore_resources, hxl_update, **kwargs)
+            for i, resource in enumerate(self.resources):
+                hdx.data.filestore_helper.FilestoreHelper.check_filestore_resource(resource, ignore_fields, filestore_resources, i)
+        self._save_dataset_add_filestore_resources('create', 'name', list(), filestore_resources, hxl_update, **kwargs)
         logger.info('Created %s' % self.get_hdx_url())
 
     def delete_from_hdx(self):
